#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2012 Liricus SRL <info@liricus.com.ar>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Fra


#===============================================================================
# DOCS
#===============================================================================

"""This module contains all backend job executed for socialframe

"""


#===============================================================================
# IMPORTS
#===============================================================================

import datetime

import flask
import flask_mail

from socialframe_server import core
from socialframe_server import models
from socialframe_server import settings
from socialframe_server import emailutil


#===============================================================================
# CRON JOBS
#===============================================================================

@core.job
def delete_inactive_users():
    now = datetime.datetime.now()
    delta = datetime.timedelta(seconds=settings.USER_NEVER_ACTIVE_EXPIRATION)
    query = models.User.filter(status=models.User.STATUS_INACTIVE)
    query = query.filter(status_datetime__lte=now - delta) 
    for iuser in query:
        iuser.delete_instance(True)


@core.job
def suspend_user():
    now = datetime.datetime.now()
    delta = datetime.timedelta(seconds=settings.USER_UNLOGED_EXPIRATION)
    query = models.User.filter(status=models.User.STATUS_ACTIVE)
    query = query.filter(last_login__lte=(now - delta))
    for user in query:
        user.status = models.User.STATUS_SUSPENDED
        user.save()
        profile = models.Profile.get(user=user)
        profile.operation = models.Profile.OPERATION_REACTIVATE_USER
        profile.save()
        data = core.absurl("reactivate_user",
                           user.email, profile.operation_hash)
        langs = (user.last_know_lang or settings.DEFAULT_LANG,)
        emailutil.send_mail(settings.MAIL_SUSPEND, user, langs, data)
    

@core.job
def delete_suspended_user():
    now = datetime.datetime.now()
    delta = datetime.timedelta(seconds=settings.USER_SUSPENDED_EXPIRATION)
    query = models.User.filter(status=models.User.STATUS_SUSPENDED)
    query = query.filter(last_login__lte=(now - delta))
    for user in query:
        user.delete_instance(True)


@core.job
def send_bubble_invitations():
    now = datetime.datetime.now()
    query = models.BubbleInvitation.filter(sended=False)
    for inv in query:
        user = inv.to_user
        if user:
            data = core.absurl("join_bubble",
                               invitation.sended_by.email,
                               invitation.bubble.name,
                               invitation.invitation_hash)
            langs = (user.last_know_lang or settings.DEFAULT_LANG,)
            emailutil.send_mail(settings.MAIL_INVITE_TO_BUBBLE,
                                user, langs, data)
        else:
            user = inv.sended_by
            data = core.absurl()
            langs = (user.last_know_lang or settings.DEFAULT_LANG,)
            emailutil.send_mail(settings.MAIL_INVITE_TO_BUBBLE,
                                user, langs, data, to=inv.email,
                                fname="", lname="")
        inv.sended = True
        inv.save()


@core.job
def delete_expired_bubble_invitations():
    now = datetime.datetime.now()
    delta = datetime.timedelta(seconds=settings.BUBBLE_INVITATION_EXPIRATION)
    query = models.BubbleInvitation.filter(sended=False)
    query = query.filter(creation_datetime__lte=(now - delta))
    for inv in query:
        inv.delete_instance(True)


@core.job
def delete_sended_bubble_invitation():
    query = models.BubbleInvitation.filter(sended=True)
    for inv in query:
        inv.delete_instance(True)
